function Test_NetworkCentricDCFmodel(Nstas)

Nstas = 4;

% Scenario

sizeX=40;
sizeY=40;

% Position of the AP
scenario.x(1)=20;
scenario.y(1)=20;
scenario.z(1)=3;

% Position of the STAs
scenario.x(2)=20;
scenario.y(2)=35;
scenario.z(2)=1;

scenario.x(3)=17;
scenario.y(3)=17;
scenario.z(3)=1;

scenario.x(4)=37;
scenario.y(4)=10;
scenario.z(4)=1;

scenario.x(5)=2;
scenario.y(5)=36;
scenario.z(5)=1;


figure
scatter(scenario.x(2:Nstas+1),scenario.y(2:Nstas+1));
hold
scatter(scenario.x(1),scenario.y(1),'r^');
hold
axis([0 40 0 40]);
xlabel('x');
ylabel('y');


% Packet sizes

traffic.L(1) = 12000; 
traffic.L(2) = 12000; 
traffic.L(3) = 12000; 
traffic.L(4) = 12000; 
traffic.L(5) = 12000; 


% WIFI

wifi.CWmin(1) = 15;
wifi.CWmax(1) = 15;

wifi.CWmin(2) = 15;
wifi.CWmax(2) = 15;

wifi.CWmin(3) = 15;
wifi.CWmax(3) = 15;

wifi.CWmin(4) = 15;
wifi.CWmax(4) = 15;

wifi.CWmin(5) = 15;
wifi.CWmax(5) = 15;


for n=1:Nstas+1
    wifi.Rmax(n) = 7;
    wifi.NSS(n) = 1;
    wifi.W(n) = 80;
end

% ----------------- Model ------------------

[Results] = NetworkCentric_DCFmodel(Nstas,scenario,traffic,wifi);

% ----------------- Results -----------------

end

function [Results] = NetworkCentric_DCFmodel(Nstas,scenario,traffic,wifi)

N = Nstas+1; % Total number of nodes transmitting (+1 is the AP)

% Transmission Rates

x = scenario.x;
y = scenario.y;
z = scenario.z;

% Position of the AP
xAP = x(1);
yAP = y(1);
zAP = z(1);

% Distances from the AP

for n=1:N
    d(n) = sqrt((x(n)-xAP)^2+(y(n)-yAP)^2+(z(n)-zAP)^2); 
end

d(1)=d(2); % AP transmits always to STA 1

disp('Distance between the AP and each STA');
disp(d);

% Received Power (TMB) model

Pt = 21;
PL = 54.12 + 10.*2.06067.*log10(d) + 5.25*0.1467.*d;
Pr = Pt - PL;

disp('Received Power (dBm)');
disp(Pr);

% Rate Selection

for n=1:N
    [DBPS(n),perr(n)]=RateSelection11ax(Pr(n),wifi.W(n),wifi.NSS(n));
end

DBPS_br = min(DBPS(2:n));

% Transmission times

for n=1:N
    [Ts(n),Tc(n)] = SUTransmission11ax(DBPS(n),traffic.L(n),DBPS_br);
end
Te = 9E-6;

disp('---- Packet Durations ----');
disp(Ts);
disp(Tc);
disp(Te);

% Traffic

for n=1:N
   L(n) = traffic.L(n);
end


% DCF parameters

for n=1:N
    CWmin(n) = wifi.CWmin(n);
    CWmax(n) = wifi.CWmax(n);
    CW_stage(n)=log2((CWmax(n)+1)/(CWmin(n)+1));
    Rmax(n) = wifi.Rmax(n); % Max number of retransmissions

end

% ------------ Iterative solver ----------

MaxIter=5000;

tau = zeros(N,MaxIter+1); % Transmission prob.
p = zeros(N,MaxIter+1);  % Collision prob.
pf = zeros(N,MaxIter+1); % Failure prob.
ENtx = zeros(N,MaxIter+1); % Exp. number of tx

% Network centric
EYnc = zeros(N,MaxIter+1); % Mean. slots per tx
penc = zeros(1,MaxIter+1); % prob. empty slot
psnc = zeros(1,MaxIter+1); % prob. succ slot
ps1nc = zeros(N,MaxIter+1); % prob. succ slot node i
pcnc = zeros(1,MaxIter+1); % prob. collision slot
ETslot = zeros(1,MaxIter+1); % Mean. slot duration
Thnc = zeros(N,MaxIter+1); % Throughput per station
ESnc = zeros(N,MaxIter+1); % Mean service time per station

% Initial Values

for n=1:N
    tau(n,1)=2/(CWmin(n)+2);
    pf(n,1)=perr(n);
end

for n=1:N
    aux_p=1;
    for m=1:N
        if m~=n
            aux_p=aux_p*(1-tau(m,1));
        end
     end
     p(n,1)=1-aux_p;
end
    
% Let's go

for i=1:MaxIter
     
    % Collision probability
    
    for n=1:N
        aux_p=1;
        for m=1:N
           if m~=n
              aux_p=aux_p*(1-tau(m,i));
           end
        end
        p(n,i+1)=1-aux_p;
        
        % Improve convergence
        if(i > 4)
           p(n,i+1)=(p(n,i+1)+p(n,i)+p(n,i-1)+p(n,i-2))/4; 
        end
        
    end
    
    % Failure Probability
    
    for n=1:N
       pf(n,i+1) = p(n,i+1)+(1-p(n,i+1))*perr(n); 
    end
    
    % Number of transmissions
    for n=1:N
       ENtx(n,i+1) =  (1-pf(n,i+1)^(Rmax(n)+1))/(1-pf(n,i+1));         
    end
    
    % Transmission probability
    for n=1:N
         
        % Exponential Backoff - Average number slots
        
        if pf(n,i+1)==0.5
            
            EYnc(n,i+1) = (((CWmin(n)+1)/2)*(2^CW_stage(n)*0.5^Rmax(n)-CW_stage(n)-2)-0.5*0.5^Rmax(n)+1)/(0.5^Rmax(n)-2);
           
        else            
                     
            A = ((CWmin(n)+1)/2)*(1-pf(n,i+1))/(1-pf(n,i+1)^(Rmax(n)+1));
            B = (1-(2*pf(n,i+1))^CW_stage(n))/(1-(2*pf(n,i+1)));
            C = ((2*pf(n,i+1))^CW_stage(n))/(1-pf(n,i+1));
            D = (2^CW_stage(n)*pf(n,i+1)^(Rmax(n)+1))/(1-pf(n,i+1));
            EY = A * (B+C-D) - 1/2;
            
            EYnc(n,i+1) = A * (B+C-D) - 1/2;
                    
        end
        tau(n,i+1) = 1/(EYnc(n,i+1)+1);
       
        % Improve convergence
        if(i > 4)
            tau(n,i+1)=(tau(n,i+1)+tau(n,i)+tau(n,i-1)+tau(n,i-2))/4; 
        end
    
    end
               
    % - Empty slot -
    
    penc(i+1) = 1;
    for n=1:N
        penc(i+1)=penc(i+1)*(1-tau(n,i+1));
    end
    
    % - Successful slot
    
    for n=1:N
        aux_ps = tau(n,i+1);
        for m=1:N
            if m~=n
                aux_ps = aux_ps * (1-tau(m,i+1));
            end
        end
        ps1nc(n,i+1)=aux_ps;
        psnc(i+1)=psnc(i+1)+aux_ps;
    end
    
    % - Collision slot
    
    pcnc(i+1)=1-penc(i+1)-psnc(i+1);
    
    %disp('Channel Probabilities');
    %disp([penc(i+1) psnc(i+1) pcnc(i+1)]);
    %pause
    
    % - Average slot duration -
    
    ETslot(i+1) = penc(i+1)*Te + pcnc(i+1)*Tc(1);
    for n=1:N
       ETslot(i+1) = ETslot(i+1) + ps1nc(n,i+1)*Ts(n);  
    end
    
    %disp('Average slot duration');
    %disp(ETslot(i+1));
    %pause
    
    % - Throuhgput -
    
    for n=1:N
        Thnc(n,i+1) = ps1nc(n,i+1)*L(n)*(1-perr(n))/ETslot(i+1);
        ESnc(n,i+1) = L(n)/Thnc(n,i+1);
    end
    
    %disp('Throughput + Service Time');
    %disp([Thnc(:,i+1) ESnc(:,i+1)]);
    %pause

    % Convergence condition
    
    CC = 0;
    for n=1:N
        if(i > 100 && ceil(p(n,i).*1E9) == ceil(p(n,i+1).*1E9))
           CC = CC + 1; 
        end
    end
    
    if(CC == N)
       disp('Convergence!!');
       disp(i);
       MaxIter = i;
       break;
    end
        
end

disp(' ----------- Resuts -----------');
disp([tau(:,MaxIter+1) p(:,MaxIter+1) ps1nc(:,MaxIter+1) Thnc(:,i+1) ESnc(:,i+1)]);

% figure('pos',[400 200 400 300])
% set(gca,'fontsize',12);
% plot(p(1,1:MaxIter+1),'r-','LineWidth',1,'Markersize',4,'Color',[0.8,0.0,0.0]);
% hold
% plot(tau(1,1:MaxIter+1),'r-','LineWidth',1,'Markersize',4,'Color',[0.0,0.6,0.0]);
% hold
% xlabel('Iteration number','FontSize',10);
% ylabel('Probability','FontSize',10);
% h_legend=legend('Collision probability','Transmission Probability');
% set(h_legend,'FontSize',10);
% axis([1 100 0 1]);

% --------------------- Results

Results.RSSI = Pr;
Results.WLAN_Throughput=Thnc(:,MaxIter+1);
Results.pf = pf(:,MaxIter+1);
Results.p = p(:,MaxIter+1);
Results.DBPS = DBPS;
Results.ES = ESnc(:,MaxIter+1);

end


function [Tsucc,Tc] = SUTransmission11ax(DBPS,L,DBPS_br)

% RTS + SIFS + CTS + SIFS + Data + SIFS + ACK + DIFS + Te;

Te = 9E-6;
SIFS = 16E-6;
DIFS = 34E-6;

PHY_SU = 164E-6;
PHY_Legacy = 20E-6;

RTS = 160;
CTS = 112;

MH = 320;
SF = 16;
TB = 18;
MD = 32;

BA = 240; % Is the compressed version
BA = 432; % Per 256;

% -----------Transmission Rate -------------------
Ts = 16E-6;
Ts_legacy = 4E-6;

BitsOFDM = DBPS;
BitsOFDMLegacyRate = 48*1*(1/2); % 6 Mbps

% ------------------------------------------------

% Successful Transmission

T_RTS = PHY_Legacy + ceil((SF+RTS+TB)/BitsOFDMLegacyRate)*Ts_legacy;
T_CTS = PHY_Legacy + ceil((SF+CTS+TB)/BitsOFDMLegacyRate)*Ts_legacy;
T_BA = PHY_Legacy + ceil((SF+BA+TB)/BitsOFDMLegacyRate)*Ts_legacy;
%T_BA = PHY_SU + ceil((SF+BA+TB)/BitsOFDM)*Ts

T_DATA = PHY_SU + ceil((SF+(MH+L)+TB)/BitsOFDM)*Ts;


%T = T_RTS + SIFS + T_CTS + SIFS + T_DATA + SIFS + T_BAR + SIFS + T_BA + DIFS + Te;

Tsucc = T_RTS + SIFS + T_CTS + SIFS + T_DATA + SIFS + T_BA + DIFS + Te;

%T = T_DATA + SIFS + T_BA + DIFS + Te;

%pause

% Collision

Tc = T_RTS+SIFS+T_CTS+DIFS+Te;

%[Na NaA T]

%pause





end




function [DBPS,perr]=RateSelection11ax(Pr,B,NSS)

Pr_norm =  Pr-3*((B/20)-1); % We reduce 3dB every time we double the channel

perr=0.05;

switch logical(1)
    case Pr >= -82 && Pr < -79; 
         Ym = 1;
         Yc = 1/2;
    case Pr >= -79 && Pr < -77;  
         Ym = 2;
         Yc = 1/2;
    case Pr >= -77 && Pr < -74;  
         Ym = 2;
         Yc = 3/4;
    case Pr >= -74 && Pr < -70;  
         Ym = 4;
         Yc = 1/2;
    case Pr >= -70 && Pr < -66;  
         Ym = 4;
         Yc = 3/4;
    case Pr >= -66 && Pr < -65;  
         Ym = 6;
         Yc = 1/2;   
    case Pr >= -65 && Pr < -64;  
         Ym = 6;
         Yc = 2/3;  
    case Pr >= -64 && Pr < -59;  
         Ym = 6;
         Yc = 3/4;           
    case Pr >= -59 && Pr < -57;  
         Ym = 8;
         Yc = 3/4;   
    case Pr >= -57 && Pr < -55;  
         Ym = 8;
         Yc = 5/6;         
    case Pr >= -55 && Pr < -53;  
         Ym = 10;
         Yc = 3/4;
    case Pr >= -53;  
         Ym = 10;
         Yc = 5/7;                  
    otherwise
         Ym = 1;
         Yc = 1/2;
         perr=0.9999;
end

[Ydsb]=DataSubcarriers11ax(B);
  
DBPS = NSS * Ym * Yc * Ydsb;

%if(Pr < -82)
%    disp([Pr Ym Yc perr]);
%    pause
%end

%disp('DBPS');
%disp([Pr_norm Ym Yc Ydsb DBPS]);


end

function [Ydsb]=DataSubcarriers11ax(B)

switch B
    case 2.5 
        Ydsb = 24;
    case 5 
        Ydsb = 48;
    case 10 
         Ydsb = 102;    
    case 20 
         Ydsb = 234;
    case 40
         Ydsb = 468;    
    case 80
         Ydsb = 980;
    case 160
         Ydsb = 1960;

    otherwise
        Ydsb = 234;
end

end



